# Reading Between the Dies: Cross-SLR Covert Channels on Multi-Tenant Cloud FPGAs

This code accompanies the following peer-reviewed paper:

I. Giechaskiel, K. B. Rasmussen, and J. Szefer, &ldquo;Reading Between the Dies:
Cross-SLR Covert Channels on Multi-Tenant Cloud FPGAs,&rdquo;
IEEE International Conference on Computer Design (ICCD), Abu Dhabi, United
Arab Emirates, November 2019.

Any updates will be posted on [this CASLAB page](https://caslab.csl.yale.edu/code/slr-covert-channel/).

## Setup

This guide assumes that you have already launched an
[AWS F1](https://aws.amazon.com/ec2/instance-types/f1/), and are connected
to it over SSH. The code has been tested with a v1.7.0
[FPGA Developer AMI](https://aws.amazon.com/marketplace/pp/B06VVYBLZZ), against
release v1.4.12 of the [FPGA HDK and SDK](https://github.com/aws/aws-fpga)
in the *us-east-1* region:

```sh
git clone -b v1.4.12 https://github.com/aws/aws-fpga/
source aws-fpga/sdk_setup.sh
```

## Data Collection

We have made four Amazon FPGA Images (AFIs) public:

1. `agfi-040599953ebb7fdee` corresponds to the default experimental setup image,
   with latch-based receiver and transmitter ROs. The 12 transmitters use
   500 ROs each, and are placed on SLR 0. The 5 receivers use 5 ROs each,
   and are placed on SLR 1.
2. `agfi-00200c7bf7eae2d32` is like (1), but instead uses flip-flop-based ROs.
3. `agfi-0326ccb201e393f2f` is like (1), but instead uses 2000 transmitter ROs,
   and moves the receiver to SLR 2.
4. `agfi-0ca2c835edaccb19a` is like (3), but instead uses flip-flop-based ROs.

To take measurements using a given AFI image, please run:

```sh
sudo python covert_channel.py -a/--agfi <agfi>
```

You can also specify any of the following optional arguments:

```
  -s/--slot         <slot_id> # FPGA slot ID for f1.4xlarge and f1.16xlarge instances
  -o/--outdir       <outdir>  # Output directory. By default measurements
                              # are saved in the cross_slr_<agfi>_<slot> directory
  -t/--trigbit      <t>       # Each measurement lasts 2**t clock cycles
  -p/--pattern      <pattern> # 32-bit transmission pattern
  -m/--measurements <meas>    # Number of measurements for the whole pattern
  -r/--repetitions  <rep>     # Number of repetitions for the experiment
  -k/--mask         <mask>    # 12-bit mask for the transmitters to enable
  -l/--loglevel     {DEBUG,INFO,WARNING,ERROR,CRITICAL}
                              # Logging level
  -w/--overwrite              # Whether to overwrite any existing measurements
```

## Data Analysis

The data collection script will create a folder (if it doesn't already exist)
containing CSV files `0.csv` up to `<#rep-1>.csv` with the raw RO measurements.
It will also save a `parameters.json` file specifying the experimental parameters.

To analyze the data, use the following command, pointing it to the measurement
folder:

```sh
sudo python csv_parser.py -i/--indir <indir>
```

This command will create a `summary.json` file with the number of errors
for any Manchester-encoded transmissions, as well as per-RO statistics for
when the transmitters are enabled and disabled.

